"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _embarkI18n = require("embark-i18n");

/* global module require */
const ListConfigs = require('./listConfigs.js');

const FunctionConfigs = require('./functionConfigs.js');

class SpecialConfigs {
  constructor(embark, options) {
    var _context, _context2, _context3, _context4, _context5, _context6;

    this.logger = embark.logger;
    this.events = embark.events;
    this.buildDir = options.buildDir;
    this.embark = embark;
    this.config = embark.config;
    this.listConfigs = new ListConfigs(embark);
    this.functionConfigs = new FunctionConfigs(embark);
    this.events.setCommandHandler('deployment:contract:address', (0, _bind.default)(_context = this.executeAddressHandlerForContract).call(_context, this));
    this.embark.registerActionForEvent('deployment:deployContracts:beforeAll', (0, _bind.default)(_context2 = this.beforeAllDeployAction).call(_context2, this));
    this.embark.registerActionForEvent('deployment:deployContracts:afterAll', (0, _bind.default)(_context3 = this.afterAllDeployAction).call(_context3, this));
    this.embark.registerActionForEvent("deployment:contract:deployed", (0, _bind.default)(_context4 = this.doOnDeployAction).call(_context4, this));
    this.embark.registerActionForEvent("deployment:contract:shouldDeploy", (0, _bind.default)(_context5 = this.deployIfAction).call(_context5, this));
    this.embark.registerActionForEvent('deployment:contract:beforeDeploy', (0, _bind.default)(_context6 = this.beforeDeployAction).call(_context6, this));
  }

  async executeAddressHandlerForContract(params, cb) {
    return this.functionConfigs.executeContractAddressHandler(params.contract, cb);
  }

  async beforeAllDeployAction(_params, cb) {
    if (typeof this.config.contractsConfig.beforeDeploy !== 'function') {
      return this.listConfigs.beforeAllDeployAction(cb);
    }

    return this.functionConfigs.beforeAllDeployAction(cb);
  }

  async afterAllDeployAction(_params, cb) {
    if (typeof this.config.contractsConfig.afterDeploy !== 'function') {
      return this.listConfigs.afterAllDeployAction(cb);
    }

    return this.functionConfigs.afterAllDeployAction(cb);
  }

  async beforeDeployAction(params, cb) {
    if (typeof params.contract.beforeDeploy !== 'function') {
      return this.listConfigs.beforeDeployAction(params, cb);
    }

    return this.functionConfigs.beforeDeployAction(params, cb);
  }

  async doOnDeployAction(params, cb) {
    let contract = params.contract;

    if (!contract.onDeploy || contract.deploy === false) {
      return cb();
    }

    if (!contract.silent) {
      this.logger.info((0, _embarkI18n.__)('executing onDeploy commands'));
    }

    if (typeof contract.onDeploy === 'function') {
      return this.functionConfigs.doOnDeployAction(contract, cb);
    }

    return this.listConfigs.doOnDeployAction(contract, cb);
  }

  async deployIfAction(params, cb) {
    let cmd = params.contract.deployIf;

    if (!cmd) {
      return cb(null, params);
    }

    if (typeof cmd === 'function') {
      return this.functionConfigs.deployIfAction(params, cb);
    }

    return this.listConfigs.deployIfAction(params, cb);
  }

}

module.exports = SpecialConfigs;
//# sourceMappingURL=index.js.map