"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _parseInt2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/parse-int"));

var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/starts-with"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/promise"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _embarkI18n = require("embark-i18n");

const async = require('async');

const stringReplaceAsync = require('string-replace-async');

const {
  callbackify
} = require('util');

class ListConfigs {
  constructor(embark) {
    this.embark = embark;
    this.events = embark.events;
    this.logger = embark.logger;
    this.config = embark.config;
  }

  beforeAllDeployAction(cb) {
    return cb();
  }

  async afterAllDeployAction(cb) {
    let afterDeployCmds = this.config.contractsConfig.afterDeploy || [];
    async.mapLimit(afterDeployCmds, 1, (cmd, nextMapCb) => {
      var _context;

      async.waterfall([next => {
        this.replaceWithAddresses(cmd, next);
      }, (0, _bind.default)(_context = this.replaceWithENSAddress).call(_context, this)], nextMapCb);
    }, (err, onDeployCode) => {
      if (err) {
        this.logger.trace(err);
        return cb(new Error("error running afterDeploy"));
      }

      this.runOnDeployCode(onDeployCode, cb);
    });
  }

  async doOnDeployAction(contract, cb) {
    let onDeployCmds = contract.onDeploy;
    async.mapLimit(onDeployCmds, 1, (cmd, nextMapCb) => {
      var _context2;

      async.waterfall([next => {
        this.replaceWithAddresses(cmd, next);
      }, (0, _bind.default)(_context2 = this.replaceWithENSAddress).call(_context2, this)], (err, code) => {
        if (err) {
          this.logger.error(err.message || err);
          return nextMapCb(); // Don't return error as we just skip the failing command
        }

        nextMapCb(null, code);
      });
    }, (err, onDeployCode) => {
      if (err) {
        return cb(new Error("error running onDeploy for " + contract.className.cyan));
      }

      this.runOnDeployCode(onDeployCode, cb, contract.silent);
    });
  }

  async deployIfAction(params, cb) {
    let cmd = params.contract.deployIf;
    this.events.request('runcode:eval', cmd, (err, result) => {
      if (err) {
        this.logger.error(params.contract.className + ' deployIf directive has an error; contract will not deploy');
        this.logger.error(err.message || err);
        params.shouldDeploy = false;
      } else if (!result) {
        this.logger.info(params.contract.className + ' deployIf directive returned false; contract will not deploy');
        params.shouldDeploy = false;
      }

      cb(null, params);
    });
  }

  async beforeDeployAction(_params, cb) {
    return cb();
  }

  runOnDeployCode(onDeployCode, callback, silent) {
    var _context3, _context4;

    const logFunction = silent ? (0, _bind.default)(_context3 = this.logger.trace).call(_context3, this.logger) : (0, _bind.default)(_context4 = this.logger.info).call(_context4, this.logger);
    async.each(onDeployCode, (cmd, eachCb) => {
      if (!cmd) {
        return eachCb();
      }

      logFunction("==== executing: " + cmd);
      this.events.request('runcode:eval', cmd, err => {
        var _context5;

        if (err && (0, _indexOf.default)(_context5 = err.message).call(_context5, "invalid opcode") >= 0) {
          this.logger.error('the transaction was rejected; this usually happens due to a throw or a require, it can also happen due to an invalid operation');
        }

        eachCb(err);
      });
    }, callback);
  }

  replaceWithENSAddress(cmd, callback) {
    const replaceWithENSAddress = cmd => {
      let regex = /\'[a-zA-Z0-9.]+\.eth\'/g;
      return stringReplaceAsync.seq(cmd, regex, ensDomain => {
        ensDomain = (0, _slice.default)(ensDomain).call(ensDomain, 1, ensDomain.length - 1);
        return new _promise.default((resolve, reject) => {
          this.events.request("namesystem:resolve", ensDomain, (err, address) => {
            if (err) {
              return reject(new Error(err));
            }

            address = `'${address}'`;
            return resolve(address);
          });
        });
      });
    };

    if (callback) {
      return callbackify(replaceWithENSAddress)(cmd, callback);
    }

    return replaceWithENSAddress(cmd);
  }

  replaceWithAddresses(cmd, callback) {
    const replaceWithAddresses = cmd => {
      let regex = /\$\w+\[?\d?\]?/g;
      return stringReplaceAsync.seq(cmd, regex, (match, index) => {
        return new _promise.default((resolve, reject) => {
          if ((0, _startsWith.default)(match).call(match, '$accounts')) {
            let accountIndex = cmd.substring(index + 10, index + 12);
            accountIndex = (0, _parseInt2.default)(accountIndex, 10);
            return this.events.request('blockchain:getAccounts', (err, accounts) => {
              if (err) {
                return reject('Error getting accounts: ' + err.message || err);
              }

              if (!accounts[accountIndex]) {
                return reject((0, _embarkI18n.__)('No corresponding account at index %d', accountIndex));
              }

              resolve(accounts[accountIndex]);
            });
          }

          let referedContractName = (0, _slice.default)(match).call(match, 1);
          this.events.request('contracts:contract', referedContractName, (_err, referedContract) => {
            if (!referedContract) {
              this.logger.error(referedContractName + ' does not exist');
              this.logger.error("error running cmd: " + cmd);
              return reject(new Error("ReferedContractDoesNotExist"));
            }

            if (referedContract && referedContract.deploy === false) {
              this.logger.error(referedContractName + " exists but has been set to not deploy");
              this.logger.error("error running cmd: " + cmd);
              return reject(new Error("ReferedContracSetToNotdeploy"));
            }

            if (referedContract && !referedContract.deployedAddress) {
              this.logger.error("couldn't find a valid address for " + referedContractName + ". has it been deployed?");
              this.logger.error("error running cmd: " + cmd);
              return reject(new Error("ReferedContractAddressNotFound"));
            }

            return resolve(referedContract.deployedAddress);
          });
        });
      });
    };

    if (callback) {
      return callbackify(replaceWithAddresses)(cmd, callback);
    }

    return replaceWithAddresses(cmd);
  }

}

module.exports = ListConfigs;
//# sourceMappingURL=listConfigs.js.map